<?php


namespace app\cbcrm\controller\admin\store;


use app\cbcrm\controller\BaseAdmin;
use app\cbcrm\model\admin\store\ConfigModel;
use app\cbcrm\model\admin\store\StoreModel;
use app\cbcrm\model\config\SettingModel;
use think\Exception;
use think\facade\Db;
use think\Request;

class Store extends BaseAdmin
{
    protected $model = null;

    public function __construct(Request $request)
    {
        parent::__construct($request);
        $this->model = new StoreModel();

    }

    public function index()
    {
        $params = [];
        $params['name'] = $this->request->get('name');
        $params['start_at'] = $this->request->get('start_at');
        $params['end_at'] = $this->request->get('end_at');
        $params['cascader'] = $this->request->get('cascader/a');
        $list = $this->model->getList($params);
        foreach ($list as $row) {
            $row->user =  Db::table('kt_base_user')->where('id',$row->wid)->value('un');
        }
        $domain_bind = config('config.domain_bind');
        $this->success('返回成功', compact('list','domain_bind'));
    }

    public function setting()
    {
        $wid = $this->request->post('wid', 0, 'intval') ?: $this->error('商户id参数错误');
        $is_default = $this->request->post('is_default', 0, 'intval');
        $cascader = $this->request->post('cascader/a');
        $province = null;
        $city = null;
        if (isset($cascader[0]) && $cascader[0]) {
            $province = $cascader[0];
        };
        if (isset($cascader[1]) && $cascader[1]) {
            $city = $cascader[1];
        };
        $row = (new ConfigModel())->where('wid', $wid)->find();
        Db::startTrans();
        try {
            if ($is_default == 1) {
                // 其他全部设置为非主站
                (new ConfigModel())->where('1=1')->update(['is_default'=>0]);
            }
            if ($row) {
                $result = $row->save(['is_default' => $is_default, 'province' => $province, 'city' => $city]);
            } else {
                $result = (new ConfigModel())->save(['wid' => $wid, 'is_default' => $is_default, 'province' => $province, 'city' => $city]);
            }
            Db::commit();
        }catch (Exception $e){
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('设置成功');
        }else{
            $this->error('设置失败');
        }
    }

    public function domainBing ()
    {
        $domain_bind = config('config.domain_bind');

        $config = config('config');
        $config['domain_bind'] = !$domain_bind;

        @file_put_contents(app_path() . 'config' . DIRECTORY_SEPARATOR . 'config.php', '<?php' . "\n\nreturn " . var_export_short($config) . ";\n");
        $appConfig = config('app');
        if ($config['domain_bind']) {

            $domainList = (new SettingModel())->where('name','domain')->where('group','pc')->where('value','<>','')->column('value');
            foreach ($domainList as $domain) {
                if ($domain != $this->request->host()) {
                    $appConfig['domain_bind'][$domain] = app('http')->getName();
                }
            }
        }else{
            $appConfig['domain_bind'] = [];
        }

        @file_put_contents(root_path('config') . DIRECTORY_SEPARATOR . 'app.php', '<?php' . "\n\nreturn " . var_export_short($appConfig) . ";\n");


       $this->success();
    }

    public function importData()
    {
        $import_table = $this->request->post('table/a');
        $source = $this->request->post('source');
        $target = $this->request->post('target');
        if (!$import_table) {
            $this->error('请选择要导入的内容');
        }
        Db::startTrans();
        try {
            // 导入演示数据
            if ($source == 'demo') {
                // 导入演示数据
                $this->importDemoData($import_table,$target);
            } else {
                $this->importStoreData($import_table,$source,$target);
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        $this->success('导入成功');
    }

    /*
     * 导入其他商户数据
     * */
    protected function importStoreData($import_table,$source,$target)
    {
        foreach ($import_table as $table) {
            switch ($table) {
                case 'product':
                    $categoryData = Db::table('cb_crm_goods_category')->where('wid',$source)->select()->toArray();
                    $categoryIds = [];
                    foreach ($categoryData as $category) {
                        $category_id = $category['id'];
                        $parent_id = 0;
                        if ($category['parent_id']!=0 && isset($categoryIds[$category['parent_id']])) {
                            $parent_id = $categoryIds[$category['parent_id']];
                        }
                        $category['parent_id'] = $parent_id;
                        $category['wid'] = $target;
                        unset($category['id']);
                        $categoryIds[$category_id] = Db::table('cb_crm_goods_category')->insertGetId($category);
                    }
                    $goodsData = Db::table('cb_crm_goods')->where('wid',$source)->select()->toArray();
                    $goodsIds = [];
                    foreach ($goodsData as $goods) {
                        if (isset($categoryIds[$goods['category_id']])) {
                            $goods_id = $goods['id'];
                            $goods['category_id'] = $categoryIds[$goods['category_id']];
                            unset($goods['id']);
                            $goods['wid'] = $target;
                            $goodsIds[$goods_id] = Db::table('cb_crm_goods')->insertGetId($goods);
                        }
                    }
                    $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id','IN',array_keys($goodsIds))->select()->toArray();
                    $groupIds = [];
                    $specValueIds = [];
                    foreach ($goodsSpec as $spec) {
                        if (isset($goodsIds[$spec['goods_id']])) {
                            $spec['goods_id'] = $goodsIds[$spec['goods_id']];
                            if ($spec['spec_value_ids']) {
                                // 查询源商户的规格值
                                $spec_value = Db::table('cb_crm_goods_spec_value')->where('id','IN',$spec['spec_value_ids'])->select()->toArray();
                                $spec_value_ids = [];
                                foreach ($spec_value as $value) {
                                    $id = $value['id'];
                                    unset($value['id']);
                                    if (!isset($specValueIds[$id])) {
                                        if (!isset($groupIds[$value['spec_group_id']])) {
                                            // 源规格组
                                            $spec_group = Db::table('cb_crm_goods_spec_group')->where('id',$value['spec_group_id'])->field('name,create_time')->find();
                                            // 新增目标商户的规格组
                                            $group_id = Db::table('cb_crm_goods_spec_group')->insertGetId($spec_group);
                                            $groupIds[$value['spec_group_id']] = $group_id;
                                        }else{
                                            $group_id =  $groupIds[$value['spec_group_id']];
                                        }
                                        $value['spec_group_id'] = $group_id;
                                        $spec_value_ids[] = $value_id = Db::table('cb_crm_goods_spec_value')->insertGetId($value);
                                        $specValueIds[$id]  = $value_id;
                                    }
                                }
                                if ($specValueIds) {
                                    $old_spec_value_ids = explode(',',$spec['spec_value_ids']);
                                    foreach ($old_spec_value_ids as $id) {
                                        $spec_value_ids[] = $specValueIds[$id];
                                    }
                                }
                                $spec['spec_value_ids'] = implode(',',$spec_value_ids);
                            }
                            unset($spec['id']);
                            Db::table('cb_crm_goods_spec')->insert($spec);
                        }
                    }
                    break;
                case 'trademark':
                    $this->insertData('cb_crm_tm_product',$source,$target);
                    break;
                case 'logo':
                    $this->insertData('cb_crm_logo_vi',$source,$target);
                    $this->insertData('cb_crm_logo_package',$source,$target);
                    $this->insertData('cb_crm_logo_other_service',$source,$target);
                    break;
                case 'copyright':
                    $sourceData = Db::table('cb_crm_copyright')->where('wid',$source)->select()->toArray();
                    $copyrightIds = [];
                    foreach ($sourceData as $key=>$item) {
                        $id = $item['id'];
                        $item['wid'] = $target;
                        unset($item['id']);
                        $copyrightIds[$id] = Db::table('cb_crm_copyright')->insertGetId($item);
                    }
                    $specData = Db::table('cb_crm_copyright_sepc')->where('copyright_id','IN',array_keys($copyrightIds))->select()->toArray();
                    foreach ($specData as $k=>$spec) {
                        unset($spec['id']);
                        if (isset($copyrightIds[$spec['copyright_id']])) {
                            $spec['copyright_id'] = $copyrightIds[$spec['copyright_id']];
                            Db::table('cb_crm_copyright_sepc')->insert($spec);
                        }
                    }
                    break;
                case 'nav':
                    $this->insertData('cb_crm_nav',$source,$target);
                    break;
                case 'block':
                    $this->insertData('cb_crm_block',$source,$target);
                    break;
            }
        }
    }

    /*
     * 导入演示数据
     * */
    protected function importDemoData($import_table,$target)
    {
        foreach ($import_table as $table) {
            switch ($table) {
                case 'product':
                    $categoryData = $this->getSqlList('goods_category.sql', $target);
                    $categoryIds = [];
                    foreach ($categoryData as $category) {
                        preg_match_all('/VALUES \((.*)\)/', $category, $cateMatches);
                        $values = explode(',', $cateMatches[1][0]);
                        $id = (int)$values[0];
                        $old_parent_id = (int)$values[1];
                        $sql = str_replace("`id`, ", '', $category);
                        if ($old_parent_id != 0 && isset($categoryIds[$old_parent_id])) {
                            $parent_id = $categoryIds[$old_parent_id];
                            $sql = str_replace("({$id}, {$old_parent_id},", "({$parent_id},", $sql);// 去除ID和父类ID
                        } else {
                            $sql = str_replace("({$id}, ", '(', $sql);// 去除ID
                        }
                        Db::execute($sql);
                        $categoryIds[$id] = Db::table('cb_crm_goods_category')->order('id', 'desc')->value('id');
                    }
                    $goodsData = $this->getSqlList('goods.sql', $target);
                    $goodsIds = [];
                    foreach ($goodsData as $goods) {
                        preg_match_all('/VALUES \((.*)\)/', $goods, $goodsMatches);
                        $values = explode(',', $goodsMatches[1][0]);
                        $goods_id = (int)$values[0];
                        $goods_category_id = (int)$values[1];
                        if ($categoryIds[$goods_category_id]) {
                            $sql = str_replace("`id`, ", '', $goods);
                            $sql = str_replace("({$goods_id}, {$goods_category_id},", "({$categoryIds[$goods_category_id]},", $sql);
                            Db::execute($sql);
                            $goodsIds[$goods_id] = Db::table('cb_crm_goods')->order('id', 'desc')->value('id');
                        }
                    }
                    $specData = $this->getSqlList('goods_spec.sql', $target);
                    foreach ($specData as $spec) {
                        preg_match_all('/VALUES \((.*)\)/', $spec, $specMatches);
                        $values = explode(',', $specMatches[1][0]);
                        $goods_id = (int)$values[0];
                        if (isset($goodsIds[$goods_id])) {
                            $sql = str_replace("({$goods_id},", "({$goodsIds[$goods_id]},", $spec);
                            Db::execute($sql);
                        }
                    }
                    break;
                case 'trademark':
                    $data = $this->getSqlList('trademark.sql', $target);
                    foreach ($data as $item) {
                        Db::execute($item);
                    }
                    break;
                case 'logo':
                    $data = $this->getSqlList('logo.sql', $target);
                    foreach ($data as $item) {
                        Db::execute($item);
                    }
                    break;
                case 'copyright':
                    $list = $this->getSqlList('copyright.sql', $target);
                    $copyrightIds = [];
                    foreach ($list as $item) {
                        preg_match_all('/VALUES \((.*)\)/', $item, $matches);
                        $values = explode(',', $matches[1][0]);
                        $id = (int)$values[0];
                        $sql = str_replace("`id`, ", '', $item);
                        $sql = str_replace("({$id}, ", '(', $sql);
                        Db::execute($sql);
                        $copyrightIds[$id] = Db::table('cb_crm_copyright')->order('id', 'desc')->value('id');
                    }
                    $speclist = $this->getSqlList('copyright_spec.sql', $target);
                    foreach ($speclist as $spec) {
                        preg_match_all('/VALUES \((.*)\)/', $spec, $specMatches);
                        $values = explode(',', $specMatches[1][0]);
                        $id = (int)$values[0];
                        if (isset($copyrightIds[$id])) {
                            $copyright_id = $copyrightIds[$id];
                            $sql = str_replace("({$id}, ", "({$copyright_id}, ", $spec);
                            Db::execute($sql);
                        }

                    }
                    break;
                case 'nav':
                    $data = $this->getSqlList('nav.sql', $target);
                    foreach ($data as $item) {
                        Db::execute($item);
                    }
                    break;
                case 'block':
                    $data = $this->getSqlList('block.sql', $target);
                    foreach ($data as $item) {
                        Db::execute($item);
                    }
                    break;
            }
        }
    }

    protected function getSqlList($file,$wid)
    {
        $dataBasePath = app()->getAppPath().'data'.DIRECTORY_SEPARATOR;
        $sql = file_get_contents($dataBasePath.$file);
        $sql = str_replace("'wid'",$wid,$sql);
        $list = explode("\n", $sql);
        $data = [];
        foreach ($list as $key=>$item) {
            $item = trim($item);
            if ($item) {
                array_push($data,$item);
            }
        }
        return $data;
    }

    protected function insertData($table,$source,$target)
    {
        $sourceData = Db::table($table)->where('wid',$source)->select()->toArray();
        foreach ($sourceData as $key=>$item) {
            unset($sourceData[$key]['id']);
            $sourceData[$key]['wid'] = $target;
        }
        Db::table($table)->insertAll($sourceData);
    }
}
