<?php


namespace app\cbcrm\controller\site;

use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\article\ArticleModel;
use app\cbcrm\model\article\NoticeModel;
use app\cbcrm\model\goods\CategoryModel;
use app\cbcrm\model\goods\GoodsModel;
use think\Collection;
use think\facade\View;

class Index extends BaseSite
{
    public function index()
    {
        // 商品分类
        $categoryList = (new CategoryModel())
            ->where('wid',$this->wid)
            ->where('status',1)
            ->order('sort desc, id desc')
            ->field('id,name,parent_id')
            ->select()
            ->toArray();

        $goodsModel = new GoodsModel();

        $categoryData = new Collection($categoryList);

        foreach ($categoryList as $key=>$item) {
            $categoryList[$key]['children'] = [];
        }

        foreach ($categoryList as $key=>$category) {
            if ($category['parent_id'] == 0) {
                // 获取下级分类
                $categoryIds = getChildrenIds($category['id'],$categoryData,[]);
                // 包含自身
                array_push($categoryIds,$category['id']);
                $goodsList  = $goodsModel
                    ->where('wid',$this->wid)
                    ->where('status',1)
                    ->where('category_id','IN',$categoryIds)
                    ->field('id,name,images,goods_price_min,goods_price_max,line_price_min,line_price_max,stock_total,url')
                    ->order('sort desc,sales_actual desc,id desc')
                    ->limit(0,8)
                    ->select();
                foreach ($goodsList as $k=>$goods) {
                    if ($goods->images) {
                        $images = explode(',',$goods->images);
                        $goodsList[$k]['image'] = cdnurl($images[0],true);
                    }
                }
                if(count($goodsList)>0){
                    $categoryList[$key]['goodsList'] = $goodsList;
                }else{
                    $categoryList[$key]['goodsList'] = [];
                }
                foreach ($categoryData as $item) {
                    if($item['parent_id'] == $category['id']){
                        $categoryList[$key]['children'][] = $item;
                    }
                }
            }else{
                unset($categoryList[$key]);
            }
        }

        $categoryList = array_values($categoryList);

        // 新闻模块
        $newsCategory = (new \app\cbcrm\model\article\CategoryModel())
            ->where('wid',$this->wid)
            ->where('status',1)
            ->limit(0,3)
            ->order('sort desc,id asc')
            ->select();

        foreach ($newsCategory as $key=>$item) {
            $newsCategory[$key]['newsList'] = (new ArticleModel)
                ->where('category_id',$item['id'])
                ->where([['wid','=',$this->wid],['status','=',1]])
                ->order('sort desc,id desc')
                ->limit(0,5)
                ->select();
        }

        // 通知公告
        $noticeList = (new NoticeModel())->where('wid',$this->wid)->where('status','1')->order('sort desc,id desc')->limit(0,5)->select();

        View::assign('noticeList',$noticeList);

        View::assign('categoryGoodsData',$categoryList);

        View::assign('newsCategory',$newsCategory);

        return View::fetch('cbcrm@site/index');
    }
}
