<?php


namespace app\cbcrm\controller\site;


use app\cbcrm\controller\BaseSite;
use app\cbcrm\model\config\DeliveryModel;
use app\cbcrm\model\config\SettingModel;
use app\cbcrm\model\goods\CategoryModel;
use app\cbcrm\model\goods\GoodsModel;
use app\cbcrm\model\goods\OrderDetailModel;
use app\cbcrm\model\goods\OrderModel;
use app\cbcrm\model\market\CouponModel;
use app\cbcrm\model\member\GradeModel;
use app\cbcrm\model\member\MemberModel;
use app\cbcrm\services\PayService;
use app\cbcrm\services\SmsServices;
use dh2y\qrcode\QRcode;
use EasyWeChat\Factory;
use think\Exception;
use think\facade\Db;
use think\facade\Log;
use think\facade\View;

class Product extends BaseSite
{
    protected $auth = null;

    public function __construct(\think\App $app)
    {
        parent::__construct($app);
        View::assign('title','产品中心');
    }

    public function index()
    {

        $category_id = $this->request->get('category_id',0,'intval');
        $keyword = $this->request->get('keyword');
        $orderby = $this->request->get('orderby','sort desc,id asc');

        $where = [
            ['wid', '=', $this->wid],
            ['status', '=', 1]
        ];

        if ($category_id) {

            $categoryList = (new CategoryModel())
                ->where('wid',$this->wid)
                ->where('status',1)
                ->order('sort desc, id desc')
                ->field('id,name,parent_id')
                ->select();
            $categoryIds = getChildrenIds($category_id,$categoryList,[]);
            array_push($categoryIds,$category_id);
            if ($categoryIds) {
                $where[] = ['category_id','IN',$categoryIds];
            }
        }
        if ($keyword) {
            $where[] = ['name','LIKE',"%{$keyword}%"];
        }
        if ($orderby == 'sale') {
            $orderby = 'sales_actual desc';
        }
        if ($orderby == 'price') {
            $orderby = 'goods_price_min asc';
        }
        // 商品分类
        $goodsParentCateogry = (new CategoryModel())
            ->where('wid', $this->wid)
            ->where('status', 1)
            ->where('parent_id', 0)
            ->order('sort desc,id asc')
            ->select();

        $goodsData = (new GoodsModel())->getList($where, 16,$orderby);

        $goodsList = $goodsData->items();

        $page = $goodsData->render();

        View::assign(compact('goodsParentCateogry', 'goodsList', 'page'));

        return View::fetch('cbcrm@site/product/index');
    }

    public function detail()
    {
        $id = $this->request->get('id', 0, 'intval');

        $row = (new GoodsModel())->getDetail($id, $this->wid);
        if ($row->url) {
            header("Location: " . $row->url);
        }
        // 服务推荐
        $resList = (new GoodsModel())
            ->where('wid', $this->wid)
            ->where('category_id', $row->category_id)
            ->where('id', '<>', $row->id)
            ->order('sales_actual', 'desc')
            ->limit(0, 4)
            ->select();
        foreach ($resList as $key => $item) {
            $resList[$key]['image'] = isset($item['images']) ? cdnurl(explode(',', $item['images'])[0]) : '';
        }

        View::assign(compact('row', 'resList'));

        return View::fetch('cbcrm@site/product/detail');
    }

    public function order()
    {
        $goods_id = $this->request->post('goods_id');
        $buy_num = $this->request->post('number', 1);
        $spec_id = $this->request->post('sku_id');// 获取商品信息
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }
        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();
        if (!$goodsSpec) {
            $this->error('购买的商品规格不存在');
        }
        if ($goodsSpec['stock_num'] < $buy_num) {
            $this->error('购买的商品规格数量不足');
        }

        if ($goodsSpec['image']) {
            $goodsSpec['image'] = cdnurl($goodsSpec['image']);
        } else {
            $goodsSpec['image'] = $goodsInfo->goods_images[0]['preview_url'];
        }

        $userInfo = session('memberInfo');

        // 收货地址
        $addressWhere = [
            ['user_address.user_id', '=', $userInfo['id']]
        ];
        $address = Db::table('cb_crm_user_address')
            ->alias('user_address')
            ->join(['cb_crm_area' => 'province'], 'user_address.province_id = province.id', 'left')
            ->join(['cb_crm_area' => 'city'], 'user_address.city_id = city.id', 'left')
            ->join(['cb_crm_area' => 'region'], 'user_address.region_id = region.id', 'left')
            ->where($addressWhere)
            ->order('user_address.create_time', 'desc')
            ->field('user_address.*,province.name as province,city.name as city,region.name as region')
            ->find();

        $goodsInfo->buy_num = $buy_num;

        $today = date('Y-m-d');
        // 可用优惠券
        $whereStr = "`wid` = {$this->wid} AND `status` = 1 
        AND (`apply_range` = 1 OR FIND_IN_SET('{$goodsInfo->id}',`apply_range_config`)) 
        AND (`expire_type` = 1 OR (`start_time` <= '{$today}' AND `end_time` >= '{$today}'))
        AND (`total_num` = -1 OR `total_num` > `receive_num`)";
        $couponList = (new CouponModel())->where($whereStr)->select();
        foreach ($couponList as $key => $item) {
            $couponList[$key]['shortname'] = $item->getAttr('name');
            $couponList[$key]['is_receive'] = false;
            if ($item->coupon_type == 1) {
                $couponList[$key]['shortname'] = '满' . $item->min_price . '减' . $item->reduce_price;
            } elseif ($item->coupon_type == 2) {
                $couponList[$key]['shortname'] = '满' . $item->min_price . '元' . $item->discount . '折';
            }

            $user_coupon = Db::table('cb_crm_user_coupon')->where('user_id', $userInfo['id'])->where('coupon_id', $item->id)->find();
            if ($user_coupon) {
                $couponList[$key]['is_receive'] = true;
                if ($user_coupon['is_use']) {
                    unset($couponList[$key]);
                }
            }

        }

        // 用户token
        $token = session('memberToken');

        View::assign(compact('goodsInfo', 'goodsSpec', 'address', 'token','couponList'));

        return View::fetch('cbcrm@site/product/order');
    }

    public function createOrder()
    {
        $this->auth = $this->auth();
        $goods_id = $this->request->post('goods_id');
        $buy_num = $this->request->post('buy_num', 1);
        $spec_id = $this->request->post('spec_id');
        $coupon_id = $this->request->post('coupon_id');
        $address_id = $this->request->post('address_id');
        $score = $this->request->post('score', 0);// 使用积分
        $params = $this->request->post();

        // 获取商品信息
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }
        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();
        if (!$goodsSpec) {
            $this->error('购买的商品规格不存在');
        }
        if ($goodsSpec['stock_num'] < $buy_num) {
            $this->error('购买的商品规格库存不足');
        }

        $addressInfo = Db::table('cb_crm_user_address')->where('id', $address_id)->where('user_id', $this->auth->id)->find();
        if (!$addressInfo) {
            $this->error('收货地址不存在');
        }

        // 商品总价
        $total_price = $goodsSpec['goods_price'] * $buy_num;
        $order_price = $total_price;
        // 订单价格 = (商品价格 * 商品数量）- 优惠券优惠金额 - 会员折扣 - 积分抵扣金额

        // 优惠劵抵扣
        $coupon_money = 0;
        if ($coupon_id) {
            $couponInfo = (new CouponModel())->getDetail($coupon_id, $this->wid);
            // 验证优惠券是否有效
            if ($couponInfo) {
                $userCoupon = Db::table('cb_crm_user_coupon')->where('user_id', $this->auth()->id)->where('coupon_id', $coupon_id)->find();
                if (!$userCoupon) {
                    $this->error('未领取到该优惠券');
                }
                // 有效期 expire_type： 1 = 领取后生效  expire_type： 2 = 固定时间
                if ($userCoupon['expire_type'] == 1 && strtotime("+{$userCoupon['expire_day']} days", strtotime($userCoupon['create_time'])) < time()) {
                    $this->error('所使用的优惠券已到期');
                }
                if ($userCoupon['expire_type'] == 2 && (strtotime($userCoupon['start_time']) > time() || strtotime($userCoupon['end_time']) < time())) {
                    $this->error('所使用的优惠券不在有效期内');
                }
                // 适用范围
                if ($userCoupon['apply_range'] == 2) {
                    $applyGoodsIdsArr = explode(',', $userCoupon['apply_range_config']);
                    if (!in_array($goods_id, $applyGoodsIdsArr)) {
                        $this->error('该商品不在优惠劵使用范围内');
                    }
                }
                // 优惠券类型:1=满减券,2=折扣券
                if ($userCoupon['min_price'] > 0 && $total_price < $userCoupon['min_price']) {
                    $this->error('商品总价未达到优惠劵最低消费金额');
                }
                // 开始抵扣优惠劵
                if ($userCoupon['coupon_type'] == 1) {
                    // 满减
                    $order_price = $order_price - $userCoupon['reduce_price'];
                    $coupon_money = $userCoupon['reduce_price'];
                } else {
                    // 折扣券
                    $order_price = $order_price * ($userCoupon['discount'] / 10);
                    $coupon_money = $total_price - $order_price;
                }

            } else {
                $this->error('所选的优惠券无效');
            }
        }
        // 会员等级抵扣 是否开启会员折扣(1开启 0关闭)
        $is_user_grade = 0; // 是否存在会员等级折扣
        $grade_ratio = 0; // 会员折扣比例
        $grade_goods_price = 0; // 会员折扣的商品单价
        $grade_total_money = 0; // 会员折扣的总额差
        if ($goodsInfo->is_enable_grade == 1) {
            // 会员折扣设置(0默认等级折扣 1单独设置折扣)
            if ($goodsInfo->is_alone_grade == 0) {
                // 当前下单用户会员等级可折扣
                $grade_ratio = (new GradeModel())->where('wid', $this->wid)->where('id', $this->auth->grade_id)->value('discount');
            } elseif ($goodsInfo->is_alone_grade == 1) {
                if (is_array($goodsInfo->alone_grade_equity)) {
                    $alone_grade_equity = $goodsInfo->alone_grade_equity;
                } else {
                    $alone_grade_equity = json_decode($goodsInfo->alone_grade_equity, true);
                }

                $key = 'grade_id:' . $this->auth->grade_id;
                $grade_ratio = isset($alone_grade_equity[$key]) ? $alone_grade_equity[$key] : 0;

            }
            if ($grade_ratio > 0) {
                $is_user_grade = 1;
                $grade_goods_price = $goodsSpec['goods_price'] * ($grade_ratio / 10);
                $grade_total_money = number_format(($goodsSpec['goods_price'] * $buy_num) - (($goodsSpec['goods_price'] * $buy_num) * ($grade_ratio / 10)), 2);
                $order_price = $order_price - $grade_total_money;
            }
        }
        // 会员积分抵扣 优先判断系统是否开启积分抵扣，再判断商品单独设置的积分抵扣
        $score_money = 0;
        $scoreConfig = (new SettingModel())->getConfigGroup('score', $this->wid);
        if ($scoreConfig['is_shopping_discount'] == 1 && $goodsInfo->is_points_discount == 1 && $score) {
            if ($total_price < $scoreConfig['full_order_price']) {
                $this->error('当前订单金额未满足积分使用条件');
            }
            // 可抵扣金额
            $scoreDiscount = $score_money = $score * $scoreConfig['discount_ratio'];
            $maxDiscount = $total_price * ($scoreConfig['max_money_ratio'] / 100);
            if ($scoreDiscount > $maxDiscount) {
                $this->error('抵扣金额超出允许范围内');
            };
            // 抵扣金额
            $order_price = $order_price - $scoreDiscount;
        }

        // 运费
        $deliveryMoney = 0;

        // 满额包邮
        $fullFreeConfig = (new SettingModel())->getConfigGroup('full_free', $this->wid);
        $isFreeDelivery = false; // 是否包邮
        if ($fullFreeConfig['is_open'] == 1 && $goodsInfo->type == 1) {
            $excludedGoodsIds = json_decode($fullFreeConfig['excludedGoodsIds'], true);
            $excludedRegions = json_decode($fullFreeConfig['excludedRegions'], true);

            if (!in_array($goods_id, $excludedGoodsIds) && !in_array($addressInfo['city'], $excludedRegions['cityIds']) && $total_price > $fullFreeConfig['money']) {
                $isFreeDelivery = true;
            }
        }
        if ($goodsInfo->type == 1 && $goodsInfo->delivery_id && !$isFreeDelivery) {
            $deliveryInfo = (new DeliveryModel())->with(['rule'])->where('id', $goodsInfo->delivery_id)->find();
            $deliveryRules = $deliveryInfo->rule;

            // 可配送区域
            $deliveryRegion = [];
            $ruleIndex = false;
            foreach ($deliveryRules as $key => $rule) {
                $region = json_decode($rule->region, true);
                $deliveryRegion = array_merge($deliveryRegion, $region);
                if (in_array($addressInfo['province_id'], $region) || in_array($addressInfo['city_id'], $region)) {
                    $ruleIndex = $key;
                }
            }
            if ($ruleIndex === false) {
                $this->error('当前收货地址不在配送范围');
            }
            // 计费方式:1=按件,2=按重
            if ($deliveryInfo->method == 1) {
                if ($buy_num <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首件价格 +（（总数 - 首件数量） / 续件数量） * 续件价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + ($buy_num - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            } else {
                if (($goodsSpec->goods_weight * $buy_num) <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首重价格 +（（总商品重量 - 首重） / 续重） * 续重价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + (($goodsSpec->goods_weight * $buy_num) - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            }
        }


        // 积分赠送 订单付款金额(100.00元) * 积分赠送比例(100%) = 实际赠送的积分(100积分) 订单的运费不参与积分赠送
        $scoreGift = 0;
        if ($scoreConfig['is_shopping_gift'] == 1 && $goodsInfo->is_points_gift == 1) {
            $scoreGift = $order_price * ($scoreConfig['gift_ratio'] / 100);
        }

        $order_no = date('YmdHis') . rand(10000, 99999);
        // 实际需支付金额 = 订单价格加运费
        $pay_price = $order_price + $deliveryMoney;
        if ($pay_price < 0.01) {
            $this->error('订单价格异常');
        }

        $orderData = [
            'wid' => $this->wid,
            'user_id' => $this->auth->id,
            'order_no' => $order_no,
            'total_price' => $total_price, // 商品总金额(不含优惠折扣) 选择规格 * 购买数量
            'order_price' => $order_price,
            'coupon_id' => $coupon_id,
            'coupon_money' => $coupon_money,
            'score_money' => $score_money,
            'score_num' => $score,
            'score_bonus' => (int)$scoreGift,
            'pay_price' => $pay_price,
            'pay_status' => 0,
            'express_price' => $deliveryMoney,
            'order_status' => 1,
            'platform' => 'pc',
            'create_time' => date('Y-m-d H:i:s'),
            'update_time' => date('Y-m-d H:i:s')
        ];

        // 2.插入订单详情表
        $orderDetail = [
            'wid' => $this->wid,
            'user_id' => $this->auth->id,
            'goods_id' => $goods_id,
            'goods_name' => $goodsInfo->getAttr('name'),
            'image' => $goodsSpec['image'] ? $goodsSpec['image'] : $goodsInfo->goods_images[0]['url'],
            'deduct_stock_type' => $goodsInfo->deduct_stock_type,
            'spec_type' => $goodsInfo->spec_type,
            'spec_id' => $goodsSpec['id'],
            'goods_sku_id' => $goodsSpec['spec_value_ids'],
            'goods_props' => $goodsSpec['spec_attr'],
            'content' => $goodsInfo->content,
            'goods_price' => $goodsSpec['goods_price'],
            'line_price' => $goodsSpec['line_price'],
            'goods_weight' => $goodsSpec['goods_weight'],
            'is_user_grade' => $is_user_grade,
            'grade_ratio' => $grade_ratio,
            'grade_goods_price' => $grade_goods_price,
            'grade_total_money' => $grade_total_money,
            'coupon_money' => $coupon_money,
            'score_money' => $score_money,
            'socre_num' => $score,
            'score_bonus' => (int)$scoreGift,
            'total_num' => $buy_num,
            'total_price' => $total_price,
            'total_pay_price' => $pay_price,
            'create_time' => date('Y-m-d H:i:s')
        ];
        $params['order_no'] = $order_no;

        $orderInfo = Db::table('cb_crm_order')->where('order_no',$order_no)->find();

        if ($orderInfo) {
            return redirect(prefix_url(true).'/member/order/index');
        }

        $token = session('memberToken');

        View::assign(compact('orderData','orderDetail','params','token'));

        return View::fetch('cbcrm@site/product/order_pay');
    }

    public function payOrder(){
        $this->auth = $this->auth();
        $goods_id = $this->request->post('goods_id');
        $buy_num = $this->request->post('buy_num', 1);
        $spec_id = $this->request->post('spec_id');
        $coupon_id = $this->request->post('coupon_id');
        $address_id = $this->request->post('address_id');
        $order_no = $this->request->post('order_no');
        $platform = $this->request->post('platform', 'wechat');
        $score = $this->request->post('score', 0);// 使用积分
        $pay_type = $this->request->post('pay_type', 'wechat'); // 支付方式
        // 获取商品信息
        $goodsInfo = (new GoodsModel())->getDetail($goods_id, $this->wid);
        if (!$goodsInfo || $goodsInfo->status != 1) {
            $this->error('购买的商品不存在');
        }
        $goodsSpec = Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->find();
        if (!$goodsSpec) {
            $this->error('购买的商品规格不存在');
        }
        if ($goodsSpec['stock_num'] < $buy_num) {
            $this->error('购买的商品规格库存不足');
        }

        $addressInfo = Db::table('cb_crm_user_address')->where('id', $address_id)->where('user_id', $this->auth->id)->find();
        if (!$addressInfo) {
            $this->error('收货地址不存在');
        }

        // 商品总价
        $total_price = $goodsSpec['goods_price'] * $buy_num;
        $order_price = $total_price;
        // 订单价格 = (商品价格 * 商品数量）- 优惠券优惠金额 - 会员折扣 - 积分抵扣金额
        // 优惠劵抵扣
        $coupon_money = 0;
        if ($coupon_id) {
            $couponInfo = (new CouponModel())->getDetail($coupon_id, $this->wid);
            // 验证优惠券是否有效
            if ($couponInfo) {
                $userCoupon = Db::table('cb_crm_user_coupon')->where('user_id', $this->auth->id)->where('coupon_id', $coupon_id)->find();
                if (!$userCoupon) {
                    $this->error('未领取到该优惠券');
                }
                if($userCoupon['is_use'] == 1){
                    $this->error('该优惠券已被使用');
                }
                // 有效期 expire_type： 1 = 领取后生效  expire_type： 2 = 固定时间
                if ($userCoupon['expire_type'] == 1 && strtotime("+{$userCoupon['expire_day']} days", strtotime($userCoupon['create_time'])) < time()) {
                    $this->error('所使用的优惠券已到期');
                }
                if ($userCoupon['expire_type'] == 2 && (strtotime($userCoupon['start_time']) > time() || strtotime($userCoupon['end_time']) < time())) {
                    $this->error('所使用的优惠券不在有效期内');
                }
                // 适用范围
                if ($userCoupon['apply_range'] == 2) {
                    $applyGoodsIdsArr = explode(',', $userCoupon['apply_range_config']);
                    if (!in_array($goods_id, $applyGoodsIdsArr)) {
                        $this->error('该商品不在优惠劵使用范围内');
                    }
                }
                // 优惠券类型:1=满减券,2=折扣券
                if ($userCoupon['min_price'] > 0 && $total_price < $userCoupon['min_price']) {
                    $this->error('商品总价未达到优惠劵最低消费金额');
                }
                // 开始抵扣优惠劵
                if ($userCoupon['coupon_type'] == 1) {
                    // 满减
                    $order_price = $order_price - $userCoupon['reduce_price'];
                    $coupon_money = $userCoupon['reduce_price'];
                } else {
                    // 折扣券
                    $order_price = $order_price * ($userCoupon['discount'] / 10);
                    $coupon_money = $total_price - $order_price;
                }

            } else {
                $this->error('所选的优惠券无效');
            }
        }
        // 会员等级抵扣 是否开启会员折扣(1开启 0关闭)
        $is_user_grade = 0; // 是否存在会员等级折扣
        $grade_ratio = 0; // 会员折扣比例
        $grade_goods_price = 0; // 会员折扣的商品单价
        $grade_total_money = 0; // 会员折扣的总额差
        if ($goodsInfo->is_enable_grade == 1) {
            // 会员折扣设置(0默认等级折扣 1单独设置折扣)
            if ($goodsInfo->is_alone_grade == 0) {
                // 当前下单用户会员等级可折扣
                $grade_ratio = (new GradeModel())->where('wid', $this->wid)->where('id', $this->auth->grade_id)->value('discount');
            } elseif ($goodsInfo->is_alone_grade == 1) {
                if (is_array($goodsInfo->alone_grade_equity)) {
                    $alone_grade_equity = $goodsInfo->alone_grade_equity;
                } else {
                    $alone_grade_equity = json_decode($goodsInfo->alone_grade_equity, true);
                }

                $key = 'grade_id:' . $this->auth->grade_id;
                $grade_ratio = isset($alone_grade_equity[$key]) ? $alone_grade_equity[$key] : 0;

            }
            if ($grade_ratio > 0) {
                $is_user_grade = 1;
                $grade_goods_price = $goodsSpec['goods_price'] * ($grade_ratio / 10);
                $grade_total_money = number_format(($goodsSpec['goods_price'] * $buy_num) - (($goodsSpec['goods_price'] * $buy_num) * ($grade_ratio / 10)), 2);
                $order_price = $order_price - $grade_total_money;
            }
        }
        // 会员积分抵扣 优先判断系统是否开启积分抵扣，再判断商品单独设置的积分抵扣
        $score_money = 0;
        $scoreConfig = (new SettingModel())->getConfigGroup('score', $this->wid);
        if ($scoreConfig['is_shopping_discount'] == 1 && $goodsInfo->is_points_discount == 1 && $score) {
            if ($total_price < $scoreConfig['full_order_price']) {
                $this->error('当前订单金额未满足积分使用条件');
            }
            // 可抵扣金额
            $scoreDiscount = $score_money = $score * $scoreConfig['discount_ratio'];
            $maxDiscount = $total_price * ($scoreConfig['max_money_ratio'] / 100);
            if ($scoreDiscount > $maxDiscount) {
                $this->error('抵扣金额超出允许范围内');
            };
            // 抵扣金额
            $order_price = $order_price - $scoreDiscount;
        }

        // 运费
        $deliveryMoney = 0;

        // 满额包邮
        $fullFreeConfig = (new SettingModel())->getConfigGroup('full_free', $this->wid);
        $isFreeDelivery = false; // 是否包邮
        if ($fullFreeConfig['is_open'] == 1 && $goodsInfo->type == 1) {
            $excludedGoodsIds = json_decode($fullFreeConfig['excludedGoodsIds'], true);
            $excludedRegions = json_decode($fullFreeConfig['excludedRegions'], true);

            if (!in_array($goods_id, $excludedGoodsIds) && !in_array($addressInfo['city'], $excludedRegions['cityIds']) && $total_price > $fullFreeConfig['money']) {
                $isFreeDelivery = true;
            }
        }
        if ($goodsInfo->type == 1 && $goodsInfo->delivery_id && !$isFreeDelivery) {
            $deliveryInfo = (new DeliveryModel())->with(['rule'])->where('id', $goodsInfo->delivery_id)->find();
            $deliveryRules = $deliveryInfo->rule;

            // 可配送区域
            $deliveryRegion = [];
            $ruleIndex = false;
            foreach ($deliveryRules as $key => $rule) {
                $region = json_decode($rule->region, true);
                $deliveryRegion = array_merge($deliveryRegion, $region);
                if (in_array($addressInfo['province_id'], $region) || in_array($addressInfo['city_id'], $region)) {
                    $ruleIndex = $key;
                }
            }
            if ($ruleIndex === false) {
                $this->error('当前收货地址不在配送范围');
            }
            // 计费方式:1=按件,2=按重
            if ($deliveryInfo->method == 1) {
                if ($buy_num <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首件价格 +（（总数 - 首件数量） / 续件数量） * 续件价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + ($buy_num - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            } else {
                if (($goodsSpec->goods_weight * $buy_num) <= $deliveryRules[$ruleIndex]['first_unit']) {
                    $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                } else {
                    // 首重价格 +（（总商品重量 - 首重） / 续重） * 续重价格
                    if ($deliveryRules[$ruleIndex]['additional'] > 0) {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'] + (($goodsSpec->goods_weight * $buy_num) - $deliveryRules[$ruleIndex]['first_unit']) / $deliveryRules[$ruleIndex]['additional'] * $deliveryRules[$ruleIndex]['additional_fee'];
                    } else {
                        $deliveryMoney = $deliveryRules[$ruleIndex]['first_fee'];
                    }
                }
            }
        }


        // 积分赠送 订单付款金额(100.00元) * 积分赠送比例(100%) = 实际赠送的积分(100积分) 订单的运费不参与积分赠送
        $scoreGift = 0;
        if ($scoreConfig['is_shopping_gift'] == 1 && $goodsInfo->is_points_gift == 1) {
            $scoreGift = $order_price * ($scoreConfig['gift_ratio'] / 100);
        }

        // 实际需支付金额 = 订单价格加运费
        $pay_price = $order_price + $deliveryMoney;
        if ($pay_price < 0.01) {
            $this->error('订单价格异常');
        }
        $result = false;
        $payData = false;
        Db::startTrans();
        try {
            /*
             * 1.插入订单表
             * 2.插入订单详情表
             * 3.插入订单地址表
             * 4.下单减库存
             * 5.修改优惠券使用状态
             * 6.扣除用户使用的积分
             * */


            // 1.插入订单表
            $orderData = [
                'wid' => $this->wid,
                'user_id' => $this->auth->id,
                'order_no' => $order_no,
                'total_price' => $total_price, // 商品总金额(不含优惠折扣) 选择规格 * 购买数量
                'order_price' => $order_price,
                'coupon_id' => $coupon_id,
                'coupon_money' => $coupon_money,
                'score_money' => $score_money,
                'score_num' => $score,
                'score_bonus' => (int)$scoreGift,
                'pay_price' => $pay_price,
                'buyer_remark' => '',
                'pay_status' => 0,
                'express_price' => $deliveryMoney,
                'order_status' => 1,
                'platform' => 'PC',
                'create_time' => date('Y-m-d H:i:s'),
                'update_time' => date('Y-m-d H:i:s')
            ];

            // 虚拟物品，默认已发货
            if ($goodsInfo->type == 0 && $pay_type == 'balance') {
                $orderData['delivery_status'] = 1;
                $orderData['delivery_time'] = date('Y-m-d H:i:s');
            }

            $order_id = (new OrderModel())->insertGetId($orderData);

            // 2.插入订单详情表
            $orderDetal = [
                'wid' => $this->wid,
                'order_id' => $order_id,
                'user_id' => $this->auth->id,
                'goods_id' => $goods_id,
                'goods_name' => $goodsInfo->getAttr('name'),
                'image' => $goodsSpec['image'] ? $goodsSpec['image'] : $goodsInfo->goods_images[0]['url'],
                'deduct_stock_type' => $goodsInfo->deduct_stock_type,
                'spec_type' => $goodsInfo->spec_type,
                'spec_id' => $goodsSpec['id'],
                'goods_sku_id' => $goodsSpec['spec_value_ids'],
                'goods_props' => $goodsSpec['spec_attr'],
                'content' => $goodsInfo->content,
                'goods_price' => $goodsSpec['goods_price'],
                'line_price' => $goodsSpec['line_price'],
                'goods_weight' => $goodsSpec['goods_weight'],
                'is_user_grade' => $is_user_grade,
                'grade_ratio' => $grade_ratio,
                'grade_goods_price' => $grade_goods_price,
                'grade_total_money' => $grade_total_money,
                'coupon_money' => $coupon_money,
                'score_money' => $score_money,
                'socre_num' => $score,
                'score_bonus' => (int)$scoreGift,
                'total_num' => $buy_num,
                'total_price' => $total_price,
                'total_pay_price' => $pay_price,
                'create_time' => date('Y-m-d H:i:s')
            ];
            $result = (new OrderDetailModel())->save($orderDetal);

            // 3.插入订单地址表
            $orderAddress = [
                'wid' => $this->wid,
                'order_id' => $order_id,
                'user_id' => $this->auth->id,
                'name' => $addressInfo['name'],
                'phone' => $addressInfo['phone'],
                'province_id' => $addressInfo['province_id'],
                'city_id' => $addressInfo['city_id'],
                'region_id' => $addressInfo['region_id'],
                'address' => $addressInfo['address'],
                'create_time' => date('Y-m-d H:i:s'),
            ];
            Db::table('cb_crm_order_address')->insert($orderAddress);

            // 4.下单减库存
            if ($goodsInfo->deduct_stock_type == 1) {
                // 下单减库存
                $goodsInfo->dec('stock_total', $buy_num)->update();
                Db::table('cb_crm_goods_spec')->where('goods_id', $goods_id)->where('id', $spec_id)->dec('stock_num', $buy_num)->update();
            }

            // 5.修改优惠券使用状态
            if ($coupon_id) {
                Db::table('cb_crm_user_coupon')->where('user_id', $this->auth->id)->where('coupon_id', $coupon_id)->update(['is_use' => 1]);
            }

            // 6.扣除用户使用的积分
            if ($score) {
                (new MemberModel())->where('id', $this->auth->id)->dec('score', $score)->update();
            }

            $payService = app()->make(PayService::class);

            if ($pay_type == 'wechat') {
                // 订单微信支付
                $payData = $payService->payWechat($orderData['pay_price'],  $orderData['order_no'], $orderDetal['goods_name'],$this->wid,'/order/wid/' . $this->wid . '/platform/PC');
                if (!$payData) {
                    throw new Exception('创建订单异常');
                }
                $payData = ['code_url'=>$payData['code_url']];
                $payData['amount'] = $orderData['pay_price'];
                $payData['order_no'] = $orderData['order_no'];
            } elseif ($pay_type == 'balance') {
                // 余额支付
                $result = $payService->payBalance($orderData['pay_price'], $orderData['order_no'],$this->wid,$this->auth(),OrderModel::class,'商品');
                if ($result!==true) {
                    throw new Exception($result);
                }
                $order = (new OrderModel())->where('order_no', $orderData['order_no'])->find();
                // 修改订单支付状态
                $order->pay_status = 1;
                $order->pay_type = 1;
                $order->pay_time = date('Y-m-d H:i:s');

                $res = $order->save();

                $orderDetailList = (new OrderDetailModel())->where('order_id', $order->id)->select();
                foreach ($orderDetailList as $key => $item) {
                    // 付款减库存
                    if ($item->deduct_stock_type == 2) {
                        (new GoodsModel())->where('id', $item->goods_id)->dec('stock_total', $item->total_num)->update();
                        Db::table('cb_crm_goods_spec')->where('goods_id', $item->goods_id)->where('id', $item->spec_id)->dec('stock_num', $item->total_num)->update();
                    }
                }
            }
            Db::commit();
        } catch (Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result) {
            $this->success('订单创建成功', $payData);
        } else {
            $this->error('订单提交失败');
        }
    }
}
